/*
main.cpp

Rev 0.1, 2015-07-31
Per Magnusson, Axotron, axotron.se/blog

This code is public domain and comes with absolutely no warranty. Enjoy!

Sensor node based on an XBee ZigBee module, a FRDM-KL25Z development board 
and an SHT21 humidity/temperature sensor.

This file contains the main program that:
- sets everything up
- regularly performs the measurements
- processes ZigBee frames that the interrupt routine has stored in frame buffers
- responds to commands from other ZigBee nodes
- prints status information to the PC serial port
*/

#include "mbed.h" 
#include "serial_buffer.h"
#include "sht21.h"
#include "zigbee.h"
#include "xbee_api.h"


Serial pc(USBTX, USBRX); 
Serial xbee(PTE0, PTE1); 
DigitalOut redled(LED1); 
DigitalOut greenled(LED2); 
DigitalOut blueled(LED3);

uint8_t rssi; // Most recent RSSI value
uint8_t rssi_updated; // Whether the RSSI value was updated since last time it was read
float humidity = -1.0;
float temperature = -1000.0;
uint8_t id = 1; // ID used for TX packets


// Update the temperature and humidity variables
void update_measurements() {
    humidity = sht21_read_humidity();
    temperature = sht21_read_temperature();    
}


// Process a frame that was a response to an AT command
void handle_at_response(const uint8_t *buffer) {
    uint32_t length;
    uint8_t frame_type;
    uint8_t status;
    uint32_t at;
    
    length = zb_get_buffer_length(buffer);
    frame_type = buffer[ZB_OFFS_FRAME_TYPE];
    if (frame_type != ZB_FRAME_TYPE_AT_RESPONSE) {
        // Not the expected frame type for this handler!
        pc.printf("Incorrect frame type 0x%02x for at-response handler\n", (uint32_t)frame_type);
        return;
    }
    status = buffer[ZB_OFFS_AT_RESP_STATUS];
    if (status != 0) {
        // Status is not OK
        pc.printf("Status is not OK (0x%02x) in at-response\n", (uint32_t)status);
        return;
    }
        
    at = zb_get_at(buffer);
    if (at == ZB_ATDB) {
        // Response to ATDB command
        if (length != 6) {
            // Unexpected length of ATDB response
            pc.printf("Unexpected length (%u) of at-response\n", length);
            return;
        }
        rssi = buffer[ZB_OFFS_AT_RESP_DATA0];
        rssi_updated = 1;
        pc.printf("RSSI = -%u dBm\n", (uint32_t)rssi);
    } else {
        pc.printf("No handler for response to AT%c%c\n", 
            buffer[ZB_OFFS_AT_RESP_AT0], buffer[ZB_OFFS_AT_RESP_AT1]);
    }
}


// Send a TX packet with humidity, temperature and RSSI information
void send_humidity_temperature(const uint8_t *dest64, const uint8_t *dest16) {
    humidity_data_t data;
    
    // Make sure measurements are OK
    if(humidity < 0) {
        update_measurements();
    }
    
    data.cmd_reply[0] = '#'; // Shows what command we are responding to
    data.cmd_reply[1] = 'M';
    data.cmd_reply[2] = 'E';
    data.cmd_reply[3] = '#';
    data.sensor_data_type[0] = 'H'; // Humidity
    data.sensor_data_type[1] = 'T'; // Temperature
    data.sensor_data_type[2] = 'R'; // RSSI
    data.sensor_data_type[3] = ' ';
    data.humidity = humidity;
    data.temperature = temperature;
    data.rssi = rssi;
    data.err_cnt = 0;
    
    id++; // Increase frame ID
    if(id < 2) {
        id = 2; // Never let it be less than 2
    }
    // Send the data
    zb_send_tx_req(id, dest64, dest16, (uint8_t *)&data, sizeof(data));
}


// Send a TX packet identifying this sensor node type
void send_id(const uint8_t *dest64, const uint8_t *dest16) {
    // Data format:
    // 4 bytes - The command this is a response to, surrounded by # signs
    // 4 bytes - The node type code
    // 4 bytes - The node revision
    // ? bytes - A free text description
    char reply[] = "#ID#" "HT  " "0001" "Humidity and temperature sensor";
    
    id++; // Increase frame ID
    if(id < 2) {
        id = 2; // Never let it be less than 2
    }
    // Send the data
    // Do not include the trailing nul in the string
    zb_send_tx_req(id, dest64, dest16, (uint8_t *)&reply, sizeof(reply)-1);
}


// Process a frame containing an RX packet
void handle_rx(const uint8_t *buffer) {
    uint8_t frame_type;
    uint32_t cmd;
    uint32_t length, data_len;
    
    pc.printf("RX handler\n");
    length = zb_get_buffer_length(buffer);
    frame_type = buffer[ZB_OFFS_FRAME_TYPE];
    if (frame_type != ZB_FRAME_TYPE_RX_RECEIVED) {
        // Not the expected frame type for this handler!
        pc.printf("Incorrect frame type 0x%02x for RX handler\n", (uint32_t)frame_type);
        return;
    }

    // Ignore receive options field
    data_len = length - ZB_OFFS_RX_DATA_0 + 3;
    if (data_len < 2) {
        pc.printf("Too little data in RX packet\n");
        return;
    }
    cmd = zb_get16(buffer, ZB_OFFS_RX_DATA_0);
    if (cmd == ZB_CMD_ME) {
        // ME Measure command
        // Send a response packet with the measurement data.
        blueled = 0;
        pc.printf("Sending reply to ME\n");
        send_humidity_temperature(buffer+ZB_OFFS_RX_SRC64_0, buffer+ZB_OFFS_RX_SRC16_0);
        blueled = 1;
    } else if (cmd == ZB_CMD_ID) {
        // ID, identify command
        // Send a response 
        blueled = 0;
        pc.printf("Sending reply to ID\n");
        send_id(buffer+ZB_OFFS_RX_SRC64_0, buffer+ZB_OFFS_RX_SRC16_0);
        blueled = 1;
    } else {
        redled = 0;
        pc.printf("No handler for RX command %c%c\n", 
            buffer[ZB_OFFS_RX_DATA_0], buffer[ZB_OFFS_RX_DATA_0+1]);
        redled = 1;
    }
}


int main() { 
    uint8_t frame_type;
    Timer rssi_timer, humidity_timer, data_timer;
    
    redled = 1;    // led off
    blueled = 1;   // led off
    greenled = 0;  // led on
    wait(1);
    pc.printf("\n\nZigBee sensor node test program\n"); 
    xbee.baud(9600);
    xbee.attach(&rx_interrupt, Serial::RxIrq);

    greenled = 1; // led off

    pc.printf("Setting up I2C\n");
    while (!sht21_setup()) {
        pc.printf("SHT21 setup failed\n");
        redled = 0;
        wait(1);
    }
    redled = 1;
    pc.printf("SHT21 setup successful\n");
    
    humidity_timer.start();
    
    // Main loop, look for frames coming from the XBee module
    while (1) {
        // Check if there is any complete frame in any buffer
        for (int ii=0; ii<n_rx_buffers; ii++) {
            if (buffer_used[ii] == 2) {
                // Found a complete frame, process it
                frame_type = rx_buffer[ii][ZB_OFFS_FRAME_TYPE];
                switch (frame_type) {
                    case ZB_FRAME_TYPE_AT_RESPONSE:
                        // Response from AT command
                        handle_at_response(rx_buffer[ii]);
                        break;
                    case ZB_FRAME_TYPE_TX_RESPONSE:
                        // Response from TX command
                        pc.printf("Response from TX\n");
                        zb_send_simple_at("DB", 1); // Update RSSI
                        break;
                    case ZB_FRAME_TYPE_RX_RECEIVED:
                        // Incoming RX frame
                        handle_rx(rx_buffer[ii]);
                        break;
                }
                // The frame has been processed, mark it as empty
                buffer_used[ii] = 0;
            }
        }
        // Timer controlled activities
        if(humidity_timer.read() > 10.0) {
            // Read humidity and temperature
            greenled = 0;
            update_measurements();
            pc.printf("Relative humidity = %.2f%% ", humidity);
            pc.printf(", Temperature = %.2f C\n", temperature);
            humidity_timer.reset();
            greenled = 1;
        }
    }
}