/*
serial_buffer.cpp

Rev 0.1, 2015-07-31
Per Magnusson, Axotron, axotron.se/blog

This code is public domain and comes with absolutely no warranty. Enjoy!

Frame buffers for data from the XBee module.

rx_interrupt() is called when data arrives from the XBee serial port. The data is parsed 
and if it is found to be a valid ZigBee frame, it is stored in a free frame buffer.
The buffer is then marked as containing a valid frame to be processed (by the main loop).
*/


#include "mbed.h" 
#include "serial_buffer.h"
#include "xbee_api.h"
#include "zigbee.h"


// Frame buffers for XBee RX data - used by interrupt routines

uint8_t rx_buffer[n_rx_buffers][buffer_size+1]; // The received frames are stored here
// buffer_used[]: 
// 0 - not used
// 1 - being filled up
// 2 - filled with a frame, waiting to be processed by main loop
// (variables with static storage are initilized to all zeros)
int buffer_used[n_rx_buffers]; 
volatile int rx_ptr=0;


// Interrupt routine to read frame data from the serial port into frame buffers.
void rx_interrupt() {
    // State: 
    // 0 = waiting for start delimiter of new frame,
    // 1 = inside frame, storing it in a buffer
    // 2 = inside frame, dropping it
    static int state = 0; 
    static int current_buffer;
    static uint32_t length;
    uint8_t cur_byte;
    static uint8_t sum; // For checksum calculation
    
    while (xbee.readable()) {
        cur_byte = ((int)xbee.getc())&0xff;
        if (state == 0) {
            // Waiting for a start delimiter
            if (cur_byte != ZB_START_DELIM) {
                // Not a start delimiter. This is a bit unexpected. 
                // Just eat it and wait for the next byte.
            } else {
                // Got a start delimiter.
                // Find a buffer to store it in.
                current_buffer = -1;
                for (int ii=0; ii < n_rx_buffers; ii++) {
                    if (!buffer_used[ii]) {
                        // Found an empty buffer
                        current_buffer = ii;
                        state = 1;
                        buffer_used[ii] = 1;
                        rx_ptr = 0;
                        rx_buffer[current_buffer][rx_ptr++] = cur_byte;
                        break;
                    }
                }
                if (current_buffer < 0) {
                    // There was no free buffer!
                    // Ignore this packet and wait for a new one.
                    pc.printf("Error: Discarding frame, no free buffers.\n");
                    state = 2;
                }
            }
        } else if (state == 1) {
            // Copying a frame into a buffer
            if (rx_ptr == 1) {
                // MSB of length
                rx_buffer[current_buffer][rx_ptr++] = cur_byte;
            } else if (rx_ptr == 2) {
                // LSB of length
                rx_buffer[current_buffer][rx_ptr++] = cur_byte;
                length = zb_get_buffer_length(rx_buffer[current_buffer]);
                sum = 0;
                if (length + 4 > buffer_size) {
                    // Ooops! The frame is bigger than the buffer, this is very unexpected.
                    // We need to discard this frame.
                    pc.printf("The frame is bigger than the buffer (%u), discarding it.\n", length);
                    buffer_used[current_buffer] = 0;
                    state = 2;
                }
            } else if (length > 0) {
                // This is a byte inside the frame
                sum += cur_byte;
                rx_buffer[current_buffer][rx_ptr++] = cur_byte;
                length--;
            } else {
                // The checksum byte
                sum += cur_byte;
                rx_buffer[current_buffer][rx_ptr++] = cur_byte;
                state = 0;
                if (sum != 0xff) {
                    // Checksum error, discard frame
                    pc.printf("Frame checksum error\n");
                    buffer_used[current_buffer] = 0;
                } else {
                    // Checksum OK, mark buffer as ready to be processed
                    buffer_used[current_buffer] = 2;
                    state = 0;
                }
            }
        } else {
            // Discarding incoming frame
            if (rx_ptr==1) {
                // MSB of length
                length = ((uint32_t)cur_byte)<<8;
            } else if (rx_ptr==2) {
                // LSB of length
                length += cur_byte;
                if (length > 102) {
                    // Length is unrealistic, we do not want to wait for 
                    // so many bytes before starting to look for a new frame.
                    // Start looking for start of a new frame immediately.
                    length = 0;
                }
            } else if (length > 0) {
                // A byte inside the frame
                length--;
            } else {
                // Finally the checksum, change state
                state = 0;
            }
        }
    }
}
