/*
sht21.cpp

Rev 0.1, 2015-07-31
Per Magnusson, Axotron, axotron.se/blog

This code is public domain and comes with absolutely no warranty. Enjoy!

Functions to talk to an SHT21 humidity and temperature sensor via I2C.
Some of the code is adapted from https://developer.mbed.org/handbook/I2C
*/


#include "mbed.h"
#include "xbee_api.h"

I2C i2c(PTC11, PTC10);
 
static const int sht21_addr = 0x80;
static const uint8_t i2c_cmd_trig_t = 0xf3;
static const uint8_t i2c_cmd_trig_h = 0xf5;
static const uint8_t i2c_cmd_write_reg = 0xe6;
static const uint8_t i2c_cmd_read_reg = 0xe7;
static const uint8_t i2c_cmd_reset = 0xfe;

static uint8_t sht21_default_reg = 0;

// Initialize the SHT21 sensor. Returns true on success.
int sht21_setup() {
    char data[2];
    
    i2c.frequency(200000); // SHT21 supports up to 400 kHz
    // Soft reset
    data[0] = i2c_cmd_reset;
    if(i2c.write(sht21_addr, data, 1)) {
        pc.printf("I2C Error: Did not receive ACK on soft reset.\n");
        return 0;
    }
    wait_ms(16); // Soft reset takes less than 15 ms
    
    // Read user register to figure out the default bits
    data[0] = i2c_cmd_read_reg;
    if(i2c.write(sht21_addr, data, 1)) {
        pc.printf("I2C Error: Did not receive ACK on read reg 1.\n");
        return 0;
    }
    if(i2c.read(sht21_addr, data, 1)) {
        pc.printf("I2C Error: Did not receive ACK on read reg 2.\n");
        return 0;
    }
    // Mask out the default bit pattern for reserved bits 3, 4, 5
    sht21_default_reg = data[0] & 0x34;
    
    // Set it to maximum resolution
    data[0] = i2c_cmd_write_reg;
    data[1] = sht21_default_reg | 0x02; // max resolution, disable heater, disable OTP reload
    if(i2c.write(sht21_addr, data, 1)) {
        pc.printf("I2C Error: Did not receive ACK on write reg.\n");
        return 0;
    }
    return 1;
}    

    
// Read the humidity from the SHT21. 
// The humidity is returned as a float (%RH).
// Returns a large negative value if an error occured.
float sht21_read_humidity() {
    char data[3];
    int32_t humidity;
    
    // Read user register to figure out default bits
    data[0] = i2c_cmd_trig_h;
    if(i2c.write(sht21_addr, data, 1, true)) {
        pc.printf("I2C Error: Did not receive ACK on humidity trigger.\n");
        wait_us(20);
        i2c.stop();
        return -(1<<24);
    }
    // Need to send the stop condition a little later
    wait_us(20);
    i2c.stop();
    wait_ms(30); // Wait until we are sure the conversion is done (29 ms maximum)
    if(i2c.read(sht21_addr, data, 3)) {
        pc.printf("I2C Error: Did not receive ACK on read humidity.\n");
        return -(1<<24);
    }
    // Ignore checksum testing
    humidity = ((((int32_t)data[0])&0xff)<<8) + (data[1]&0xfc); // Combine bytes and clear status bits
    return 125.0*humidity/65536.0 - 6.0;
}


// Read the temperature from the SHT21. 
// The temperature is returned as a float (degrees C).
// Returns a large negative value if an error occured.
float sht21_read_temperature() {
    char data[3];
    int32_t temperature;
    
    // Read user register to figure out default bits
    data[0] = i2c_cmd_trig_t;
    if(i2c.write(sht21_addr, data, 1, true)) {
        pc.printf("I2C Error: Did not receive ACK on temperature trigger.\n");
        wait_us(20);
        i2c.stop();
        return -(1<<24);
    }
    // Need to send the stop condition a little later
    wait_us(20);
    i2c.stop();
    wait_ms(90); // Wait until we are sure the conversion is done (85 ms maximum)
    if(i2c.read(sht21_addr, data, 3)) {
        pc.printf("I2C Error: Did not receive ACK on read temperature.\n");
        return -(1<<24);
    }
    // Ignore checksum testing
    temperature = ((((int32_t)data[0])&0xff)<<8) + (data[1]&0xfc); // Combine bytes and clear status bits
    return 175.72*temperature/65536.0 - 46.85; 
}
    
    
