#! /usr/bin/python

"""
xbee_rx.py

Rev 0.1, 2015-07-31
Per Magnusson, Axotron, axotron.se/blog

This code is public domain and comes with absolutely no warranty. Enjoy!

Regularly send measurement requests to a remote ZigBee sensor node and display 
the data that comes back.
"""

from xbee import ZigBee
import serial
import serial.tools.list_ports
import sys
import time
import msvcrt
import struct

# Remote node address
long_addr = "\x00\x13\xA2\x00\x40\xB4\x51\x40" # Hard coded for now
short_addr = '\xff\xfe' # Use 0xfffe as 16-bit address until it becomes known


# Callback function that runs when frames are received
def receive_data(data):
    global short_addr
    if data['id'] == 'rx_explicit':
        if short_addr == '\xff\xfe' and data['source_addr_long'] == long_addr:
            short_addr = data['source_addr']
            print 'Discovered remote 16-bit address: 0x{:02x}{:02x}'.format(ord(short_addr[0]), 
                                                                            ord(short_addr[1]))
        # Transmission from a node
        if data['rf_data'][:4] == '#ID#':
            # Frame with node ID
            print '\nNode identifiaction frame'
            print 'Node type: "' + data['rf_data'][4:8] + '"'
            print 'Node revision: "' + data['rf_data'][8:12] + '"'
            print 'Node description: "' + data['rf_data'][12:] + '"'
        elif data['rf_data'][:4] == '#ME#':
            # Reply to measure command
            print 'Measurement data frame'
            if data['rf_data'][4:8] == "HTR ":
                humidity = struct.unpack('f', data['rf_data'][8:12])[0]
                temperature = struct.unpack('f', data['rf_data'][12:16])[0]
                rssi = ord(data['rf_data'][16])
                err_cnt = ord(data['rf_data'][17])
                print "Humidity: {0:.2f}%, Temperature: {1:.2f} C, RSSI: -{2:d} dBm".format(humidity, 
                                                                                            temperature, 
                                                                                            rssi)
            else:
                print 'Measurement from unknown node type "{}"'.format(data['rf_data'][4:8])
                print data
        else:
            print 'Unknown RX frame type "{}"'.format(data['rf_data'][:4])
            print data
    elif data['id'] == 'tx_status':
        # Status from a transmission
        print '\nTX status frame, fid={:d}'.format(ord(data['frame_id']))
        if ord(data['retries']) != 0:
            print 'TX Warning: {:d} retries were done'.format(ord(data['retries']))
        if ord(data['deliver_status']) != 0:
            print 'TX Warning: delivery failed 0x{:x}'.format(ord(data['deliver_status']))
        if ord(data['discover_status']) != 0:
            print 'TX Warning: discovery overhead 0x{:x}'.format(ord(data['discover_status']))
    else:
        print "\nUnknown frame type"
        print data




# Look for a COM port that might have an XBee connected
portfound = False
ports = list(serial.tools.list_ports.comports())
for p in ports:
    # The SparkFun XBee Explorer USB board uses an FTDI chip as USB interface
    print p
    if "FTDIBUS" in p[2]:
        print "Found possible XBee on " + p[0]
        if not portfound:
            portname = p[0]
            portfound = True
            print "Using " + portname + " as XBee COM port."
        else:
            print "Ignoring this port, using the first one that was found."

if portfound:
    ser = serial.Serial(portname, 9600)
else:
    sys.exit("No serial port seems to have an XBee connected.")


xbee = ZigBee(ser, callback = receive_data)
cnt = 0
cnt_me = 100
id = 2

# Request information about remote node
xbee.send("tx", data="ID", dest_addr_long=long_addr, dest_addr=short_addr)
print "Press q to quit"

# Main loop
while True:
    try:
        if msvcrt.kbhit():
            # Ctrl-C exits the whole shell in IPython on Windows, 
            # so make a press on the q-key exit the main loop.
            if msvcrt.getch() == 'q':
                print "Exiting loop"
                break
        time.sleep(0.1)
        cnt = cnt+1
        cnt_me = cnt_me+1
        if cnt >= 10:
            # Show that the program is still running
            sys.stdout.write('.')
            sys.stdout.flush()
            cnt = 0
        if cnt_me >= 100:
            # Send request for measurement data to remote node
            xbee.send("tx", data="ME", dest_addr_long=long_addr, dest_addr=short_addr, frame_id=chr(id))
            id = id+1
            if id > 255:
                id = 2
            cnt_me = 0

    except KeyboardInterrupt:
        print "Ctrl-C was pressed"
        break
        
xbee.halt()
ser.close()
