/*
zigbee.cpp

Rev 0.1, 2015-07-31
Per Magnusson, Axotron, axotron.se/blog

Functions to deal with the XBee ZigBee module.
*/

#include "xbee_api.h"
#include "zigbee.h"


// Calculate and set the checksum of the ZB buffer pointed to by buffer with the length len.
// len is the number of bytes in the buffer, including start delimiter and length.
void zb_set_checksum(uint8_t *buffer, int len) {
    uint8_t sum = 0;
    
    for(int ii = ZB_OFFS_LEN_LSB+1; ii < len; ii++) {
        sum += buffer[ii];
    }
    buffer[len] = 0xff - sum;
}

    
// Set the length field of the ZigBee buffer. 
// len is the number of bytes in the buffer, including start delimiter and length, 
// but excluding the checksum
void zb_set_len(uint8_t *buffer, uint32_t len) {
    uint32_t len_val;
    
    len_val = len - 3;
    buffer[ZB_OFFS_LEN_MSB] = len_val >> 8;
    buffer[ZB_OFFS_LEN_LSB] = len_val & 0xff;
}


// Send a buffer containing a ZigBee frame to the XBee module
void zb_send_buffer(const uint8_t *buffer) {
    uint32_t len;

    len = zb_get_buffer_length(buffer) + 4;
    pc.printf("Sending frame: ");    
    for(int ii=0; ii < len; ii++) {
        pc.printf("%02x ", (uint32_t)buffer[ii]);
        xbee.putc(buffer[ii]);
    }
    pc.printf("\n");    
}


// Send an AT command without parameter
void zb_send_simple_at(const char *at, char frame_id) {
    uint8_t buffer[8];
    uint32_t bufp = 0;
    
    buffer[0] = ZB_START_DELIM;
    buffer[ZB_OFFS_FRAME_TYPE] = ZB_FRAME_TYPE_AT;
    buffer[ZB_OFFS_FRAME_ID] = frame_id;
    buffer[ZB_OFFS_AT_CMD_AT0] = at[0];
    buffer[ZB_OFFS_AT_CMD_AT1] = at[1];
    bufp = ZB_OFFS_AT_CMD_AT1+1;
    zb_set_checksum(buffer, bufp);
    zb_set_len(buffer, bufp);
    
    zb_send_buffer(buffer);           
}


// Send an AT command with a single byte as parameter
void zb_send_at_byte_param(const char *at, uint8_t param) {
    uint8_t buffer[9];
    uint32_t bufp = 0;
    
    buffer[0] = ZB_START_DELIM;
    buffer[ZB_OFFS_FRAME_TYPE] = ZB_FRAME_TYPE_AT;
    buffer[ZB_OFFS_FRAME_ID] = 0;
    buffer[ZB_OFFS_AT_CMD_AT0] = at[0];
    buffer[ZB_OFFS_AT_CMD_AT1] = at[1];
    bufp = ZB_OFFS_AT_CMD_AT1+1;
    buffer[bufp++] = param;
    zb_set_checksum(buffer, bufp);
    zb_set_len(buffer, bufp);
    
    zb_send_buffer(buffer);           
}


// Send a transmit request frame
void zb_send_tx_req(uint8_t id, const uint8_t *dest64, const uint8_t *dest16, const uint8_t *data, int data_len) {
    uint8_t buffer[18+82]; // How much payload can there be?
    int bufp = 0;
    
    buffer[0] = ZB_START_DELIM;
    buffer[ZB_OFFS_FRAME_TYPE] = ZB_FRAME_TYPE_TX_REQ;
    buffer[ZB_OFFS_FRAME_ID] = id;
    for(int ii=0; ii<8; ii++) {
        buffer[ZB_OFFS_TX_DEST64_0+ii] = dest64[ii];
    }
    buffer[ZB_OFFS_TX_DEST16_0] = dest16[0];
    buffer[ZB_OFFS_TX_DEST16_1] = dest16[1];
    buffer[ZB_OFFS_TX_RADIUS] = 0;
    buffer[ZB_OFFS_TX_OPTIONS] = 0;
    bufp = ZB_OFFS_TX_DATA_0;
    for(int ii=0; ii<data_len; ii++) {
        buffer[bufp++] = data[ii];
    }
    zb_set_checksum(buffer, bufp);
    zb_set_len(buffer, bufp);
    zb_send_buffer(buffer);           
}


// Return two bytes (MSB and LSB) from a uint8_t array as an unsigned int
uint32_t zb_get16(const uint8_t *buffer, int start) {
    return (buffer[start]<<8) + buffer[start+1];
}


// Return the length field from a frame
int zb_get_buffer_length(const uint8_t *buffer) {
    return zb_get16(buffer, ZB_OFFS_LEN_MSB);
}


// Return the AT command field from an AT command response frame
uint32_t zb_get_at(const uint8_t *buffer) {
    return zb_get16(buffer, ZB_OFFS_AT_RESP_AT0);
}